/**
 *  @author		SVGANIMA
 *  @date		2022
 *  @copyright	GNU GPLv3
 *	@brief		Path extension providing animation of the SVG's viewbox
 */
function animatedViewbox(target) {
	this.x = null;
	this.y = null;
	this.width = null;
	this.height = null;
	
	if(target instanceof root) {
		var candidate = null;
		for(var i = 0; i < target.svgElement.children.length; i++) {
			if(target.svgElement.children[i] instanceof SVGAnimateElement &&
				target.svgElement.children[i].getAttribute('attributeName') == 'viewBox') {
					candidate = target.svgElement.children[i];
					break;
				}
		}
		
		this.x = target.svgBox[0];
		this.y = target.svgBox[1];
		this.width = target.svgBox[2];
		this.height = target.svgBox[3];
		
		this.element = document.createElementNS(svgNS, 'path');
		
		this.element.setAttribute('shape-rendering', 'crispEdges');
		
		this.element.style.stroke = 'gray';
		this.element.style.fill = 'none';
		
		this.element.setAttribute('svganima:type', "animatedViewbox");
		this.element.setAttribute('svganima:x', this.x);
		this.element.setAttribute('svganima:y', this.y);
		this.element.setAttribute('svganima:width', this.width);
		this.element.setAttribute('svganima:height', this.height);
		
		if(candidate) {
			this.element.setAttribute('svganima:keytimes', candidate.getAttribute('keyTimes'));
			this.element.setAttribute('svganima:keysplines', candidate.getAttribute('keySplines'));
			this.element.setAttribute('svganima:values', candidate.getAttribute('values'));
			this.element.setAttribute('svganima:begin', candidate.getAttribute('begin'));
			this.element.setAttribute('svganima:dur', candidate.getAttribute('dur'));
			this.element.setAttribute('svganima:repeatcount', candidate.getAttribute('repeatCount'));
			this.element.setAttribute('svganima:calcmode', candidate.getAttribute('calcMode'));
			this.element.setAttribute('svganima:additive', candidate.getAttribute('additive'));
			this.element.setAttribute('svganima:accumulate', candidate.getAttribute('accumulate'));
			this.element.setAttribute('svganima:fill', candidate.getAttribute('fill'));
			candidate.parentNode.removeChild(candidate);
			this.element.setAttribute('id', candidate.getAttribute('id'));
		} else {
			this.element.setAttribute('svganima:keytimes', "0;1");
			this.element.setAttribute('svganima:keysplines', "0 0 1 1");
			this.element.setAttribute('svganima:values', target.svgBox.join(' ')+";"+target.svgBox.join(' '));
			this.element.setAttribute('svganima:begin', "0s");
			this.element.setAttribute('svganima:dur', "10s");
			this.element.setAttribute('svganima:repeatcount', "indefinite");
			this.element.setAttribute('svganima:calcmode', "spline");
			this.element.setAttribute('svganima:additive', "replace");
			this.element.setAttribute('svganima:accumulate', "none");
			this.element.setAttribute('svganima:fill', "freeze");
			this.element.generateId(false, 'camera');
		}
		
		this.animation = document.createElementNS(svgNS, 'animate');
		this.animation.setAttribute('attributeType', 'auto');
		this.animation.setAttribute('attributeName', 'd');
		this.animation.setAttribute('svganima:lock', 'skip');	
		this.animation.generateId();
		
		this.element.appendChild(this.animation);
		
		this.getBeginList();
		this.getDur();
		this.getRepeatCount();
		this.getCalcMode();
		this.getKeyframes();
		
		this.commit(true);
		
		this.element.shepherd = this;
		
		this.refresh();
		this.adjustZoom();
		svg.svgElement.appendChild(this.element);
	}
}

animatedViewbox.prototype = Object.create(animationGroup.prototype);

animatedViewbox.prototype.refresh = function() {
	var pData = [];
		pData.push('M');
		pData.push(this.x);
		pData.push(this.y);
		pData.push('L');
		pData.push(this.x+this.width);
		pData.push(this.y);
		pData.push('L');
		pData.push(this.x+this.width);
		pData.push(this.y+this.height);
		pData.push('L');
		pData.push(this.x);
		pData.push(this.y+this.height);
		pData.push('Z');
	this.element.setAttribute('d', pData.join(' '));
	this.element.style.strokeWidth = 2/svg.zoom+"px";
}

animatedViewbox.prototype.adjustZoom = function() {
	if(svgoooActual.settings.get('canvasFrame')) {
		this.element.style.display = null;
	} else {
		this.element.style.display = 'none';
	}
	this.element.style.strokeWidth = 2/svg.zoom+"px";
}


animatedViewbox.prototype.getKeyframes = function() {
	if(this.keyframes) {
		return this.keyframes;
	}
	
	var timesArray = this.getAttribute('svganima:keytimes') ? this.getAttribute('svganima:keytimes').split(';') : [];
	var splineArray = this.getAttribute('svganima:keysplines') ? this.getAttribute('svganima:keysplines').split(';') : [];
	var valueArray = this.getAttribute('svganima:values') ? this.getAttribute('svganima:values').split(';') : [];
	var intensityArray = this.getAttribute('svganima:intensity') ? this.getAttribute('svganima:intensity').split(';') : [];
	
	this.keyframes = new keyframeList();
	
	for(var i = 0; i < timesArray.length; i++) {
		var newValue = new rectangle(valueArray[i]);
		this.keyframes.push(
			new keyframe(parseFloat(timesArray[i]),
				(splineArray[i-1] ? new spline(splineArray[i-1]) : null),
				newValue,
				(intensityArray && intensityArray[i] != null ? parseFloat(intensityArray[i]) : null)
			)
		);
	}
	return this.keyframes;
}

animatedViewbox.prototype.commit = function(noHistory) {
	if(noHistory) { this.wipe(); }
	this.getKeyframes();
	
	var out = this.element;
	var count = 0;
	var intensityChanged = false;
	
	var newTimes = this.keyframes.getTimes().join(';');
	var newSplines = this.keyframes.getSplines().join(';');
	var newIntensity = this.keyframes.getIntensity().join(';');
	var newValues = this.keyframes.getValues().join(';');
	
	var newBegin = this.getBeginList().join(';');
	var newDur = this.getDur().toString();
	
	var newRepeatCount = String(this.getRepeatCount());
	var newCalcMode = this.getCalcMode();
	var newFill = this.getFill();
	var newAdditive = this.getAdditive();
	var newAccumulate = this.getAccumulate();
	
	var histFrom = {};
	var histTo = {};
	
	if(newTimes != this.getAttribute('svganima:keytimes') && newTimes.length != 0 && this.getAttribute('svganima:keytimes')) {
		histFrom['svganima:keytimes'] = this.getAttribute('svganima:keytimes');
		histTo['svganima:keytimes'] = newTimes;
		this.setAttribute('svganima:keytimes', newTimes);
		count++;
	}
	if(newSplines != this.getAttribute('svganima:keysplines') && newSplines.length != 0 && this.getAttribute('svganima:keysplines')) {
		histFrom['svganima:keysplines'] = this.getAttribute('svganima:keysplines');
		histTo['svganima:keysplines'] = newSplines;
		this.setAttribute('svganima:keysplines', newSplines);
		count++;
	}
	
	if(newValues != this.getAttribute('svganima:values') && newValues.length != 0 && this.getAttribute('svganima:values')) {
		var oldValues = this.getAttribute('svganima:values').split(';');
		histFrom['svganima:values'] = this.getAttribute('svganima:values');
		histTo['svganima:values'] = newValues;
		this.setAttribute('svganima:values', newValues);
		
		count++;
	}
	
	if(newDur != this.getAttribute('svganima:dur') && newDur.length != 0 && this.getAttribute('svganima:dur')) {
		histFrom['svganima:dur'] = this.getAttribute('svganima:dur');
		histTo['svganima:dur'] = newDur;
		this.setAttribute('svganima:dur', newDur);
		count++;
	}
	
	if(newRepeatCount != this.getAttribute('svganima:repeatcount') && newRepeatCount.length != 0 && this.getAttribute('svganima:repeatcount')) {
		histFrom['svganima:repeatcount'] = this.getAttribute('svganima:repeatcount');
		histTo['svganima:repeatcount'] = newRepeatCount;
		this.setAttribute('svganima:repeatcount', newRepeatCount);
		count++;
	}
	if(newCalcMode != this.getAttribute('svganima:calcmode') && newCalcMode.length != 0 && this.getAttribute('svganima:calcmode')) {
		histFrom['svganima:calcmode'] = this.getAttribute('svganima:calcmode');
		histTo['svganima:calcmode'] = newCalcMode;
		this.setAttribute('svganima:calcmode', newCalcMode);
		count++;
	}
	if(newFill != this.getAttribute('svganima:fill') && newFill.length != 0 && this.getAttribute('svganima:fill')) {
		histFrom['svganima:fill'] = this.getAttribute('svganima:fill');
		histTo['svganima:fill'] = newFill;
		this.setAttribute('svganima:fill', newFill);
		count++;
	}
	if(newAdditive != this.getAttribute('svganima:additive') && newAdditive.length != 0 && this.getAttribute('svganima:additive')) {
		histFrom['svganima:additive'] = this.getAttribute('svganima:additive');
		histTo['svganima:additive'] = newAdditive;
		this.setAttribute('svganima:additive', newAdditive);
		count++;
	}
	if(newAccumulate != this.getAttribute('svganima:accumulate') && newAccumulate.length != 0 && this.getAttribute('svganima:accumulate')) {
		histFrom['svganima:accumulate'] = this.getAttribute('svganima:accumulate');
		histTo['svganima:accumulate'] = newAccumulate;
		this.setAttribute('svganima:accumulate', newAccumulate);
		count++;
	}
	
	
	if(newBegin != this.getAttribute('svganima:begin') && newBegin.length != 0 && this.getAttribute('svganima:begin')) {
		histFrom['svganima:begin'] = this.getAttribute('svganima:begin');
		histTo['svganima:begin'] = newBegin;
		this.setAttribute('svganima:begin', newBegin);
		count++;
	}
	
	// rebuilds everything for animation because reasons
	var newValues = [];
	for(var i = 0; i < this.keyframes.length; i++) {
		var currentFrame = this.keyframes.getItem(i);
		var pData = [];
			pData.push('M');
			pData.push(currentFrame.value.x);
			pData.push(currentFrame.value.y);
			pData.push('L');
			pData.push(currentFrame.value.x+currentFrame.value.width);
			pData.push(currentFrame.value.y);
			pData.push('L');
			pData.push(currentFrame.value.x+currentFrame.value.width);
			pData.push(currentFrame.value.y+currentFrame.value.height);
			pData.push('L');
			pData.push(currentFrame.value.x);
			pData.push(currentFrame.value.y+currentFrame.value.height);
			pData.push('Z');
		newValues.push(pData.join(' '));
	}
	newValues = newValues.join(';');
	
	this.animation.setAttribute('keyTimes', newTimes);
	this.animation.setAttribute('keySplines', newSplines);
	this.animation.setAttribute('values', newValues);
	this.animation.setAttribute('dur', newDur);
	this.animation.setAttribute('repeatCount', newRepeatCount);
	this.animation.setAttribute('calcMode', newCalcMode);
	this.animation.setAttribute('fill', newFill);
	this.animation.setAttribute('additive', newAdditive);
	this.animation.setAttribute('accumulate', newAccumulate);
	this.animation.setAttribute('begin', newBegin);
	var clone = this.animation.cloneNode(true);
	this.animation.parentNode.insertBefore(clone, this.animation);
	this.animation.parentNode.removeChild(this.animation);
	this.animation = clone;
	
	window.dispatchEvent(new Event('treeSeed'));
	window.dispatchEvent(new Event('rootSelect'));
	
	if(!noHistory && svg && svg.history && count > 0) {
		svg.history.add(new historyAttribute(this.element.id, histFrom, histTo, true));
	}
	
	if(svgoooActual.settings.get('previewAutorefresh')) {
		svg.previewWindow.seed(true);
	}
	
	return out;
}

animatedViewbox.prototype.transferOut = function() {
	this.getBeginList();
	this.getDur();
	this.getRepeatCount();
	this.getCalcMode();
	this.getKeyframes();
	
	var anim = this.animation.cloneNode();
		anim.setAttribute('attributeName', 'viewBox');
		anim.setAttribute('values', this.element.getAttribute('svganima:values'));
		anim.setAttribute('id', this.element.id);
		
	return anim;
}

animatedViewbox.prototype.generateAnchors = function() {
	if(svgoooManager.classes.windowAnimation.selected.length == 0) { return null; }
	
	this.getKeyframes();
	
	svg.ui.selectionBox.hide();
	
	var allAnchors = [];
	var allConnectors = [];
	var allPaths = [];

	var previousAnchors;
	var currentAnchors;
	var firstAnchors;
	var currentRectangle;
		
	var currentKeyframe;
	var previousKeyframe;
	
		
		
	var threshold = 0.01;
	
	for(var i = 0; i < svgoooManager.classes.windowAnimation.selected.length; i++) {
		currentKeyframe = this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[i]);
		
		if(currentKeyframe && previousKeyframe && currentKeyframe != previousKeyframe && previousAnchors && 
			Math.abs(currentKeyframe.value.x-previousKeyframe.value.x) < threshold && Math.abs(currentKeyframe.value.y-previousKeyframe.value.y) < threshold && 
			Math.abs(currentKeyframe.value.width-previousKeyframe.value.width) < threshold && Math.abs(currentKeyframe.value.height-previousKeyframe.value.height) < threshold
		) {
			previousAnchors.anchors[0][0].actions.move += 'this.animation.setValue('+svgoooManager.classes.windowAnimation.selected[i]+', null, this.element, true);';
			previousAnchors.anchors[0][1].actions.move += 'this.animation.setValue('+svgoooManager.classes.windowAnimation.selected[i]+', null, this.element, true);';
			continue;
		}
		
		currentRectangle = document.createElementNS(svgNS, 'rect');
		currentRectangle.setAttribute('x', currentKeyframe.value.x);
		currentRectangle.setAttribute('y', currentKeyframe.value.y);
		currentRectangle.setAttribute('width', currentKeyframe.value.width);
		currentRectangle.setAttribute('height', currentKeyframe.value.height);
		currentRectangle.setAttribute("svganima:lock", "interface");
		currentRectangle.setAttribute('style', 'fill:none;stroke:#aa0000;stroke-width:'+2/(svg.zoom)+'px');
		currentAnchors = currentRectangle.generateAnchors(true);
		
		if(firstAnchors == null) { firstAnchors = currentAnchors; }
		
		if(svgoooManager.classes.windowAnimation.selected.length != 1 && i == svgoooManager.classes.windowAnimation.selected.length-1 &&
			Math.abs(this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[i]).value.x-this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[0]).value.x) < threshold &&
			Math.abs(this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[i]).value.y-this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[0]).value.y) < threshold &&
			Math.abs(this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[i]).value.width-this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[0]).value.width) < threshold &&
			Math.abs(this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[i]).value.height-this.keyframes.getItem(svgoooManager.classes.windowAnimation.selected[0]).value.height) < threshold) {
				currentAnchors = firstAnchors;
		} else {
			allAnchors = allAnchors.concat(currentAnchors.anchors);
			allPaths.push(currentRectangle);
		}
		
		currentAnchors.anchors[0][1].constraint.flags.optional = false;
		
		for(var j = 0; j < currentAnchors.anchors[0].length; j++) {
			currentAnchors.anchors[0][j].animation = this;
			currentAnchors.anchors[0][j].actions.move += 'this.animation.setValue('+svgoooManager.classes.windowAnimation.selected[i]+', null, this.element, true);';
		}
		
		if(svgoooActual.settings.get('progressCurve') && previousAnchors && currentAnchors && currentAnchors != previousAnchors) {
			for(var j = 0; j < currentAnchors.anchors[0].length; j++) {
				var slave = new connector(currentAnchors.anchors[0][j], previousAnchors.anchors[0][j], '#00aa00');
				allConnectors.push(slave);
			}
		}
		previousAnchors = currentAnchors;
		previousKeyframe = currentKeyframe;
	}
	return { 'connectors': allConnectors, 'anchors': allAnchors, 'paths': allPaths };
}

animatedViewbox.prototype.setValue = function(index, portion, value) {
	this.getKeyframes();
	
	if(index < 0 || index >= this.keyframes.length) { throw new DOMException(1); }
	
	var val = this.keyframes.getItem(index);
	
	if(portion instanceof SVGRectElement) {
		value = portion;
		portion = null;
	}
	
	if(portion == null) {
		if(!(value instanceof SVGRectElement)) { throw new DOMException(17); }
		val.value.x = typeof value.x === 'number' ? value.x : value.x.baseVal.value;
		val.value.y = typeof value.y === 'number' ? value.y : value.y.baseVal.value;
		val.value.width = typeof value.width === 'number' ? value.width : value.width.baseVal.value;
		val.value.height = typeof value.height === 'number' ? value.height : value.height.baseVal.value;
		this.commit();
		return;
	}
	
	if(portion < 0 || portion > 4) { throw new DOMException(1); }
	
	switch(portion) {
		case 0:
			val.value.x = value;
			break;
		case 1:
			val.value.y = value;
			break;
		case 2:
			var ratio = svg.svgBox[3]/svg.svgBox[2];
			val.value.width = value;
			val.value.height = value*ratio;
			break;
		case 3:
			var ratio = svg.svgBox[2]/svg.svgBox[3];
			val.value.width = value*ratio;
			val.value.height = value;
			break;
	}
	
	this.commit();
}



