/**
 *  @author		SVGANIMA
 *  @date		2022
 *  @copyright	GNU GPLv3
 *	@brief		Picker tool
 */
function toolPicker() {
	tool.call(this);
}

toolPicker.prototype = Object.create(tool.prototype);

toolPicker.prototype.mouseClick = function(event) {
	if(!svg || !(svg instanceof root) || event.button == 2) { return; }
	
	if(event.target == svg.svgElement || svg.selected == svg.svgElement) { return; }
	
	if(event.target.original) { event.target = event.target.original; }
	
	if(svg.selected instanceof SVGAnimationElement && event.shiftKey && event.target.hasAnimation() && event.target.getAnimations()[0]) {
		svg.selected.pasteTiming(event.target.getAnimations()[0]);
		svg.select(svg.selected.commit());
		return;
	}
	
	if(svg.selected instanceof SVGAnimateMotionElement && event.target instanceof SVGPathElement) {
		// motion gets a new path if the target is a path
		svg.selected.setPath(event.target, event.shiftKey || event.ctrlKey);
		svg.selected.commit();
		return;
	}
	
	if((svg.selected.shepherd instanceof animationGroup || svg.selected instanceof SVGAnimationElement) &&
		(event.target.shepherd instanceof animationGroup || event.target.hasAnimation()) &&
		!(svg.selected instanceof SVGAnimateElement) || (svg.selected instanceof SVGAnimateElement && svgoooManager.classes.windowAnimation.selected.length == 0)) {
		
		var fr, to;
		to = svg.selected.shepherd || svg.selected;
		fr = event.target.shepherd || event.target.getAnimations()[0];
		
		to.pasteTiming(fr, true)
		svg.select(to.commit());
		
		return;
	}
	
	if(svg.selected instanceof SVGAnimateElement) {
		// attribute animation gets new values for selected animation keyframes (if applicable)
		if(svgoooManager.classes.windowAnimation.selected.length == 0) { return; }
		var val = event.target.style[svg.selected.getAttribute('attributeName')] || event.target.getAttribute(svg.selected.getAttribute('attributeName'));
		if(!val) {
			// if no values is found, reverts to the parent's default value instead
			val = svg.selected.parentNode.style[svg.selected.getAttribute('attributeName')] || svg.selected.parentNode.getAttribute(svg.selected.getAttribute('attributeName'));
		}
		for(var i = 0; i < svgoooManager.classes.windowAnimation.selected.length; i++) {
			svg.selected.setValue(svgoooManager.classes.windowAnimation.selected[i], val, true);
		}
		svg.selected.commit();
		svgoooManager.classes.windowAnimation.refreshKeyframes();
		svg.ui.edit(svg.selected);
		return;
	}
	
	var tFillColor = event.target.style.fill;
	var tFillOpacity = event.target.style.fillOpacity || 1;
	
	var trg = event.target;
	while((tFillColor == 'inherit' || tFillColor == null) && trg.parentNode && !(trg.parentNode instanceof SVGSVGElement)) {
		trg = trg.parentNode;
		tFillColor = trg.style.fill;
	}
	trg = event.target;
	while(tFillOpacity == 'inherit' && trg.parentNode && !(trg.parentNode instanceof SVGSVGElement)) {
		trg = trg.parentNode;
		tFillOpacity = trg.style.fillOpacity;
	}
	
	if(event.shiftKey) {
		svg.selected.setAttributeHistory({'stroke': tFillColor, 'strokeOpacity': tFillOpacity || 1 });
	} else {
		svg.selected.setAttributeHistory({'fill': tFillColor, 'fillOpacity': tFillOpacity || 1 });
	}
	
	// svgoooManager.classes.windowColors.refresh();
	
}

toolPicker.prototype.mouseMove = function(event) {
	if(!svg || !(svg instanceof root)) { return; }
	
	this.lastEvent = event;
	
	// cursor change
	
	
	// target or selection is svgElement - DEFAULT
	if(event.target == svg.svgElement || svg.selected == svg.svgElement) {
		svgoooManager.setCursor('url(_cursors/picker_none.png) 5 5');
	} else
	// selected (or selected shepherd) is animation
	if(svg.selected instanceof SVGAnimationElement || svg.selected.shepherd instanceof SVGAnimationElement) {
		// some keyframes are selected (copy values)
		if(svgoooManager.classes.windowAnimation.selected.length > 0) {
			// target has corresponding attribute - DEFAULT
			if(event.target.getAttribute(svg.selected.getAttribute('attributeName')) || event.target.style[svg.selected.getAttribute('attributeName')]) {
				svgoooManager.setCursor('url(_cursors/picker.png) 5 5');
			} else {
			// target has no such attribute	- NO PICK
				svgoooManager.setCursor('url(_cursors/picker_none.png) 5 5');
			}
		} else {
		// no keyframes are selected (copy timing)
			if(event.target.hasAnimation()) {
			// target has animation - TIMING
				svgoooManager.setCursor('url(_cursors/picker_time.png) 5 5');
			} else if(svg.selected instanceof SVGAnimateMotionElement && event.target instanceof SVGPathElement) {
				svgoooManager.setCursor('url(_cursors/picker_path.png) 5 5');
			} else {
			// target has no animation - NO PICK
				svgoooManager.setCursor('url(_cursors/picker_none.png) 5 5');
			}
		}
	} else {
	// selected is regular object
		if(event.shiftKey) {	// shift key - PICK STROKE
			svgoooManager.setCursor('url(_cursors/picker_stroke.png) 5 5');
		} else {	// no shift key - PICK FILL
			svgoooManager.setCursor('url(_cursors/picker_fill.png) 5 5');
		}
	}
	
	if((event.button == 1 || event.buttons == 4) && this.lastEvent) {
		var dX = Math.round((this.lastEvent.clientX - event.clientX)/svg.zoom);
		var dY = Math.round((this.lastEvent.clientY - event.clientY)/svg.zoom);
		this.lastEvent = event;
		svg.moveView(dX, dY);
		return;
	}
}

toolPicker.prototype.keyDown = function(event) {
	if(!svg) { return; }
	if(svg.selected instanceof SVGAnimateElement || svg.selected.shepherd instanceof SVGAnimateElement || !this.lastEvent || (this.lastEvent && this.lastEvent.target == svg.svgElement) || svg.selected == svg.svgElement) {
		return true;
	}
	if(event.shiftKey) {
		svgoooManager.setCursor('url(_cursors/picker_stroke.png) 5 5');
	} else {
		svgoooManager.setCursor('url(_cursors/picker_fill.png) 5 5');
	}
	return true;
}

toolPicker.prototype.keyUp = function(event) {
	if(event.shiftKey) {
		svgoooManager.setCursor('url(_cursors/picker_stroke.png) 5 5');
	} else {
		svgoooManager.setCursor('url(_cursors/picker_fill.png) 5 5');
	}
	return true;
}


